@@ -41,61 +41,87 @@ Net::Jabber::Message - Jabber Message Module
 
 =head2 Retrieval functions
 
-    $from     = $Mess->GetFrom();
-    $resource = $Mess->GetResource();
-    $id       = $Mess->GetID();
-    $type     = $Mess->GetType();
-    $subject  = $Mess->GetSubject();
-    $body     = $Mess->GetBody();
-    @body     = $Mess->GetBody("full");
-    $thread   = $Mess->GetThread();
-    $priority = $Mess->GetPriority();
-    $error    = $Mess->GetError();
-    $errType  = $Mess->GetErrorType();
-    @xTags    = $Mess->GetX();
-    @xTags    = $Mess->GetX("my:namespace");
-    @xTrees   = $Mess->GetXTrees();
-    @xTrees   = $Mess->GetXTrees("my:namespace");
+    $to         = $Mess->GetTo();
+    $toJID      = $Mess->GetTo("jid");
+    $from       = $Mess->GetFrom();
+    $fromJID    = $Mess->GetFrom("jid");
+    $etherxTo   = $Mess->GetEtherxTo();
+    $etherxFrom = $Mess->GetEtherxFrom();
+    $resource   = $Mess->GetResource();
+    $type       = $Mess->GetType();
+    $subject    = $Mess->GetSubject();
+    $body       = $Mess->GetBody();
+    @body       = $Mess->GetBody("full");
+    $thread     = $Mess->GetThread();
+    $priority   = $Mess->GetPriority();
+    $error      = $Mess->GetError();
+    $errType    = $Mess->GetErrorType();
+    @xTags      = $Mess->GetX();
+    @xTags      = $Mess->GetX("my:namespace");
+    @xTrees     = $Mess->GetXTrees();
+    @xTrees     = $Mess->GetXTrees("my:namespace");
 
-    $str      = $Mess->GetXML();
-    @message  = $Mess->GetTree();
+    $str        = $Mess->GetXML();
+    @message    = $Mess->GetTree();
+
+    $date       = $Mess->GetTimeStamp();
 
 =head2 Creation functions
 
-    $Mess->SetMessage(-TO=>"bob\@jabber.org/Working Bob",
-		      -Subject=>"Lunch",
-		      -BoDy=>"Let's go grab some lunch!",
-		      -priority=>100);
-    $Mess->SetTo("Test User");
+    $Mess->SetMessage(TO=>"bob\@jabber.org/Working Bob",
+		      Subject=>"Lunch",
+		      BoDy=>"Let's go grab some lunch!",
+		      priority=>100);
+    $Mess->SetTo("test\@jabber.org");
+    $Mess->SetFrom("me\@jabber.org");
+    $Mess->SetEtherxTo("jabber.org");
+    $Mess->SetEtherxFrom("transport.jabber.org");
+    $Mess->SetType("groupchat");
     $Mess->SetSubject("This is a test");
     $Mess->SetBody("This is a test of the emergency broadcast system...");
     $Mess->SetThread("AE912B3");
     $Mess->SetPriority(1);
 
-    $Mess->SetMessage(-to=>"bob\@jabber.org",
-                      -errortype=>"denied",
-                      -error=>"Permission Denied");
+    $Mess->SetMessage(to=>"bob\@jabber.org",
+                      errortype=>"denied",
+                      error=>"Permission Denied");
     $Mess->SetErrorType("denied");
     $Mess->SetError("Permission Denied");
 
     $X = $Mess->NewX("jabber:x:delay");
     $X = $Mess->NewX("my:namespace");
 
+    $Reply = $Mess->Reply();
+    $Reply = $Mess->Reply(template=>"client");
+    $Reply = $Mess->Reply(template=>"transport");
+
 =head1 METHODS
 
 =head2 Retrieval functions
 
-  GetTo() - returns a string with the Jabber Identifier of the 
-            person who is going to receive the <message/>.
+  GetTo()      - returns either a string with the Jabber Identifier,
+  GetTo("jid")   or a Net::Jabber::JID object for the person who is 
+                 going to receive the <message/>.  To get the JID
+                 object set the string to "jid", otherwise leave
+                 blank for the text string.
 
-  GetFrom() - returns a string with the Jabber Identifier of the 
-              person who sent the <message/>.
+  GetFrom()      -  returns either a string with the Jabber Identifier,
+  GetFrom("jid")    or a Net::Jabber::JID object for the person who
+                    sent the <message/>.  To get the JID object set 
+                    the string to "jid", otherwise leave blank for the 
+                    text string.
+
+  GetEtherxTo(string) - returns the etherx:to attribute.  This is for
+                        Transport writers who need to communicate with
+                        Etherx.
+
+  GetEtherxFrom(string) -  returns the etherx:from attribute.  This is for
+                           Transport writers who need to communicate with
+                           Etherx.
 
   GetResource() - returns a string with the Jabber Resource of the 
                   person who sent the <message/>.
 
-  GetID() - returns an integer with the id of the <message/>.
-
   GetType() - returns a string with the type <message/> this is.
 
   GetSubject() - returns a string with the subject of the <message/>.
@@ -118,47 +144,67 @@ Net::Jabber::Message - Jabber Message Module
 
   GetErrorType() - returns a string with the type of the <error/> tag.
 
-  GetX(string) - returns an array of Net::Jabber::X objects.  The string can 
-                 either be empty or the XML Namespace you are looking for.  
-                 If empty then GetX returns every <x/> tag in the 
+  GetX(string) - returns an array of Net::Jabber::X objects.  The string 
+                 can either be empty or the XML Namespace you are looking
+                 for.  If empty then GetX returns every <x/> tag in the 
                  <message/>.  If an XML Namespace is sent then GetX 
                  returns every <x/> tag with that Namespace.
 
   GetXTrees(string) - returns an array of XML::Parser::Tree objects.  The 
                       string can either be empty or the XML Namespace you 
-                      are looking for.  If empty then GetXTrees returns every 
-                      <x/> tag in the <message/>.  If an XML Namespace is 
-                      sent then GetXTrees returns every <x/> tag with that 
-                      Namespace.
+                      are looking for.  If empty then GetXTrees returns
+                      every <x/> tag in the <message/>.  If an XML
+                      Namespace is sent then GetXTrees returns every
+                      <x/> tag with that  Namespace.
 
   GetXML() - returns the XML string that represents the <message/>.
              This is used by the Send() function in Client.pm to send
              this object as a Jabber Message.
 
   GetTree() - returns an array that contains the <message/> tag
-                 in XML::Parser Tree format.
+              in XML::Parser Tree format.
+
+  GetTimeStamp() - returns a string that represents the time this message
+                   object was created (and probably received) for sending
+                   to the client.  If there is an <x/> delay tag then that
+                   time is used to show when the message was sent.
 
 =head2 Creation functions
 
-  SetMessage(to=>string,         - set multiple fields in the <message/>
-             type=>string,         at one time.  This is a cumulative
-             subject=>string,      and over writing action.  If you set
-             body=>string,         the "to" attribute twice, the second
-             thread=>string,       setting is what is used.  If you set
-             priority=>integer,    the subject, and then set the body
-             errortype=>string,    then both will be in the <message/>
-             error=>string)        tag.  For valid settings read the
-                                   specific Set functions below.
+  SetMessage(to=>string|JID,     - set multiple fields in the <message/>
+             from=>string|JID,     at one time.  This is a cumulative
+             type=>string,         and over writing action.  If you set
+             subject=>string,      the "to" attribute twice, the second
+             body=>string,         setting is what is used.  If you set
+             thread=>integer,      the subject, and then set the body
+             priority=>string,     then both will be in the <message/>
+             errortype=>string,    tag.  For valid settings read the
+             error=>string)        specific Set functions below.
 
-  SetTo(string) - sets the to attribute.  Must be a valid Jabber Identifier 
-                  or the server will return an error message.
+  SetTo(string) - sets the to attribute.  You can either pass a string
+  SetTo(JID)      or a JID object.  They must be valid Jabber 
+                  Identifiers or the server will return an error message.
                   (ie.  jabber:bob@jabber.org/Silent Bob, etc...)
 
+  SetFrom(string) - sets the from attribute.  You can either pass a string
+  SetFrom(JID)      or a JID object.  They must be valid Jabber 
+                    Identifiers or the server will return an error message.
+                    (ie.  jabber:bob@jabber.org/Silent Bob, etc...)
+
+  SetEtherxTo(string) - sets the etherx:to attribute.  This is for
+                        Transport writers who need to communicate with
+                        Etherx.
+
+  SetEtherxFrom(string) -  sets the etherx:from attribute.  This is for
+                           Transport writers who need to communicate with
+                           Etherx.
+
   SetType(string) - sets the type attribute.  Valid settings are:
 
-                    normal         defines a normal message
                     chat           defines a chat style message
+                    error          defines an error message
                     groupchat      defines a chatroom message
+                    normal         defines a normal message (default)
 
   SetSubject(string) - sets the subject of the <message/>.
 
@@ -183,9 +229,38 @@ Net::Jabber::Message - Jabber Message Module
                  namespace with the X module.  For more information
                  please read the documentation for Net::Jabber::X.
 
+  Reply(template=>string,       - creates a new Message object and
+        replytransport=>string)   populates the to/from and
+                                  etherxto/etherxfrom fields based
+                                  the value of template.  The following
+                                  templates are available:
+
+                                  client: (default)
+                                       just sets the to/from
+
+                                  transport:
+                                  transport-reply:
+                                       the transport will send the
+                                       reply to the sender
+
+                                  transport-filter:
+                                       the transport will send the
+                                       reply to the address from the
+                                       to.  ie( bob%j.org@transport.j.org
+                                       would send to bob@j.org)
+
+                                  transport-filter-reply:
+                                       the transport will send the
+                                       reply to the address from the
+                                       to.  ie( bob%j.org@transport.j.org
+                                       would send to bob@j.org) and
+                                       set the from to be 
+                                       sender@replytransport.  That
+                                       way a two way filter can occur.
+
 =head1 AUTHOR
 
-By Ryan Eatmon in December of 1999 for http://jabber.org..
+By Ryan Eatmon in May of 2000 for http://jabber.org..
 
 =head1 COPYRIGHT
 
@@ -199,7 +274,7 @@ use strict;
 use Carp;
 use vars qw($VERSION);
 
-$VERSION = "0.8.1";
+$VERSION = "1.0";
 
 sub new {
   my $proto = shift;
@@ -207,10 +282,11 @@ sub new {
   my $self = { };
   
   $self->{VERSION} = $VERSION;
+  $self->{TIMESTAMP} = &Net::Jabber::GetTimeStamp("local");
 
   bless($self, $proto);
 
-  if (@_ != ("")) {
+  if ("@_" ne ("")) {
     my @temp = @_;
     $self->{MESSAGE} = \@temp;
     my $xTree;
@@ -226,6 +302,17 @@ sub new {
 }
 
 
+##############################################################################
+#
+# GetTag - returns the Jabber tag of this object
+#
+##############################################################################
+sub GetTag {
+  my $self = shift;
+  return "message";
+}
+
+
 ##############################################################################
 #
 # GetID - returns the id of the <message/>
@@ -242,11 +329,16 @@ sub GetID {
 # GetTo - returns the Jabber Identifier of the person you are sending the
 #         <message/> to.
 #
-#
 ##############################################################################
 sub GetTo {
   my $self = shift;
-  return &Net::Jabber::GetXMLData("value",$self->{MESSAGE},"","to");
+  my ($type) = @_;
+  my $to = &Net::Jabber::GetXMLData("value",$self->{MESSAGE},"","to");
+  if ($type eq "jid") {
+    return new Net::Jabber::JID($to);
+  } else {
+    return $to;
+  }
 }
 
 
@@ -255,11 +347,40 @@ sub GetTo {
 # GetFrom - returns the Jabber Identifier of the person who sent the 
 #           <message/>
 #
-#
 ##############################################################################
 sub GetFrom {
   my $self = shift;
-  return &Net::Jabber::GetXMLData("value",$self->{MESSAGE},"","from");
+  my ($type) = @_;
+  my $from = &Net::Jabber::GetXMLData("value",$self->{MESSAGE},"","from");
+  if ($type eq "jid") {
+    return new Net::Jabber::JID($from);
+  } else {
+    return $from;
+  }
+}
+
+
+##############################################################################
+#
+# GetEtherxTo - returns the value of the etherx:to attribute in the 
+#               <message/>.
+#
+##############################################################################
+sub GetEtherxTo {
+  my $self = shift;
+  return &Net::Jabber::GetXMLData("value",$self->{MESSAGE},"","etherx:to");
+}
+
+
+##############################################################################
+#
+# GetEtherxFrom - returns the value of the etherx:from attribute in the 
+#                 <message/>.
+#
+##############################################################################
+sub GetEtherxFrom {
+  my $self = shift;
+  return &Net::Jabber::GetXMLData("value",$self->{MESSAGE},"","etherx:from");
 }
 
 
@@ -268,7 +389,6 @@ sub GetFrom {
 # GetResource - returns the Jabber Resource of the person who sent the 
 #              <message/>
 #
-#
 ##############################################################################
 sub GetResource {
   my $self = shift;
@@ -421,6 +541,25 @@ sub GetTree {
 }
 
 
+##############################################################################
+#
+# GetTimeStamp - returns a string with the time stamp of when this object
+#                was created.
+#
+##############################################################################
+sub GetTimeStamp {
+  my $self = shift;
+
+  my @xTags = $self->GetX("jabber:x:delay");
+  if ($#xTags >= 0) {
+    my $xTag = $xTags[0];
+    $self->{TIMESTAMP} = &Net::Jabber::GetTimeStamp("utcdelaylocal",$xTag->GetStamp());
+  }
+
+  return $self->{TIMESTAMP};
+}
+
+
 ##############################################################################
 #
 # SetMessage - takes a hash of all of the things you can set on a <message/>
@@ -434,6 +573,10 @@ sub SetMessage {
 
   $self->SetID($message{id}) if exists($message{id});
   $self->SetTo($message{to}) if exists($message{to});
+  $self->SetFrom($message{from}) if exists($message{from});
+  $self->SetEtherxTo($message{etherxto}) if exists($message{etherxto});
+  $self->SetEtherxFrom($message{etherxfrom}) if exists($message{etherxfrom});
+  $self->SetType($message{type}) if exists($message{type});
   $self->SetSubject($message{subject}) if exists($message{subject});
   $self->SetBody($message{body}) if exists($message{body});
   $self->SetThread($message{thread}) if exists($message{thread});
@@ -463,10 +606,64 @@ sub SetID {
 sub SetTo {
   my $self = shift;
   my ($to) = @_;
+  if (ref($to) eq "Net::Jabber::JID") {
+    $to = $to->GetJID();
+  }
   &Net::Jabber::SetXMLData("single",$self->{MESSAGE},"","",{to=>$to});
 }
 
 
+##############################################################################
+#
+# SetFrom - sets the from attribute in the <message/>
+#
+##############################################################################
+sub SetFrom {
+  my $self = shift;
+  my ($from) = @_;
+  if (ref($from) eq "Net::Jabber::JID") {
+    $from = $from->GetJID();
+  }
+  &Net::Jabber::SetXMLData("single",$self->{MESSAGE},"","",{from=>$from});
+}
+
+
+##############################################################################
+#
+# SetEtherxTo - sets the etherx:to attribute in the <message/>
+#
+##############################################################################
+sub SetEtherxTo {
+  my $self = shift;
+  my ($etherxto) = @_;
+  &Net::Jabber::SetXMLData("single",$self->{MESSAGE},"","",{"etherx:to"=>$etherxto});
+}
+
+
+##############################################################################
+#
+# SetEtherxFrom - sets the etherx:from attribute in the <message/>
+#
+##############################################################################
+sub SetEtherxFrom {
+  my $self = shift;
+  my ($etherxfrom) = @_;
+  &Net::Jabber::SetXMLData("single",$self->{MESSAGE},"","",{"etherx:from"=>$etherxfrom});
+}
+
+
+##############################################################################
+#
+# SetType - sets the type attribute in the <message/>
+#
+##############################################################################
+sub SetType {
+  my $self = shift;
+  my ($type) = @_;
+  &Net::Jabber::SetXMLData("single",$self->{MESSAGE},"","",{type=>$type});
+}
+
+
 ##############################################################################
 #
 # SetSubject - sets the subject of the <message/>
@@ -610,6 +807,71 @@ sub MergeX {
 }
 
 
+##############################################################################
+#
+# Reply - returns a Net::Jabber::Message object with the proper fields
+#         already populated for you.
+#
+##############################################################################
+sub Reply {
+  my $self = shift;
+  my %args;
+  while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
+
+  my $reply = new Net::Jabber::Message();
+
+  $reply->SetThread($self->GetThread()) if ($self->GetThread() ne "");
+  $reply->SetID($self->GetID()) if ($self->GetID() ne "");
+  $reply->SetType($self->GetType()) if ($self->GetType() ne "");
+
+
+  if (exists($args{template})) {
+    if (($args{template} eq "transport") || ($args{template} eq "transport-reply")) {
+      my $fromJID = $self->GetFrom("jid");
+
+      $reply->SetMessage(to=>$self->GetFrom(),
+			 from=>$self->GetTo(),
+			 etherxto=>$fromJID->GetServer(),
+			 etherxfrom=>$self->GetEtherxTo(),
+			);
+    } else {
+      if ($args{template} eq "transport-filter") {
+	my $toJID = $self->GetTo("jid");
+	my $fromJID = $self->GetFrom("jid");
+
+	my $filterToJID = new Net::Jabber::JID($toJID->GetUserID());
+
+	$reply->SetMessage(to=>$filterToJID,
+			   from=>$fromJID,
+			   etherxto=>$filterToJID->Server(),
+			   etherxfrom=>$fromJID->Server());
+      } else {
+	if ($args{template} eq "transport-filter-reply") {
+	  my $toJID = $self->GetTo("jid");
+	  my $fromJID = $self->GetFrom("jid");
+	  
+	  my $filterToJID = new Net::Jabber::JID($toJID->GetUserID());
+	  my $filterFromJID = new Net::Jabber::JID($fromJID->GetUserID()."\%".$fromJID->GetServer()."\@".$args{replytransport});
+	  
+	  $reply->SetMessage(to=>$filterToJID,
+			     from=>$filterFromJID,
+			     etherxto=>$filterToJID->Server(),
+			     etherxfrom=>$self->GetEtherxTo());
+	} else {
+	  $reply->SetMessage(to=>$self->GetFrom(),
+			     from=>$self->GetTo());
+	}	
+      }
+    }
+  } else {
+    $reply->SetMessage(to=>$self->GetFrom(),
+		       from=>$self->GetTo());
+  }
+
+  return $reply;
+}
+
+
 ##############################################################################
 #
 # debug - prints out the XML::Parser Tree in a readable format for debugging
