@@ -41,29 +41,36 @@ Net::Jabber::Presence - Jabber Presence Module
 
 =head2 Retrieval functions
 
-    $to       = $Pres->GetTo();
-    $from     = $Pres->GetFrom();
-    $type     = $Pres->GetType();
-    $status   = $Pres->GetStatus();
-    $priority = $Pres->GetPriority();
-    $meta     = $Pres->GetMeta();
-    $icon     = $Pres->GetIcon();
-    $show     = $Pres->GetShow();
-    $loc      = $Pres->GetLoc();
-    @xTags    = $Mess->GetX();
-    @xTags    = $Mess->GetX("my:namespace");
-    @xTrees   = $Mess->GetXTrees();
-    @xTrees   = $Mess->GetXTrees("my:namespace");
+    $to         = $Pres->GetTo();
+    $toJID      = $Pres->GetTo("jid");
+    $from       = $Pres->GetFrom();
+    $fromJID    = $Pres->GetFrom("jid");
+    $etherxTo   = $Pres->GetEtherxTo();
+    $etherxFrom = $Pres->GetEtherxFrom();
+    $type       = $Pres->GetType();
+    $status     = $Pres->GetStatus();
+    $priority   = $Pres->GetPriority();
+    $meta       = $Pres->GetMeta();
+    $icon       = $Pres->GetIcon();
+    $show       = $Pres->GetShow();
+    $loc        = $Pres->GetLoc();
+    @xTags      = $Pres->GetX();
+    @xTags      = $Pres->GetX("my:namespace");
+    @xTrees     = $Pres->GetXTrees();
+    @xTrees     = $Pres->GetXTrees("my:namespace");
 
-    $str      = $Pres->GetXML();
-    @presence = $Pres->GetTree();
+    $str        = $Pres->GetXML();
+    @presence   = $Pres->GetTree();
 
 =head2 Creation functions
 
-    $Pres->SetPresence(-TYPE=>"online",
-		       -StatuS=>"Open for Business",
-		       -iCoN=>"normal");
+    $Pres->SetPresence(TYPE=>"online",
+		       StatuS=>"Open for Business",
+		       iCoN=>"normal");
     $Pres->SetTo("bob\@jabber.org");
+    $Pres->SetFrom("jojo\@jabber.org");
+    $Pres->SetEtherxTo("jabber.org");
+    $Pres->SetEtherxFrom("transport.jabber.org");
     $Pres->SetType("unavailable");
     $Pres->SetStatus("Taking a nap");
     $Pres->SetPriority(10);
@@ -75,15 +82,34 @@ Net::Jabber::Presence - Jabber Presence Module
     $X = $Pres->NewX("jabber:x:delay");
     $X = $Pres->NewX("my:namespace");
 
+    $Reply = $Pres->Reply();
+    $Reply = $Pres->Reply(template=>"client");
+    $Reply = $Pres->Reply(template=>"transport",
+                          type=>"subscribed");
+
 =head1 METHODS
 
 =head2 Retrieval functions
 
-  GetTo() - returns a string with the Jabber Identifier of the 
-            person who is going to receive the <presence/>.
+  GetTo()      - returns either a string with the Jabber Identifier,
+  GetTo("jid")   or a Net::Jabber::JID object for the person who is 
+                 going to receive the <presence/>.  To get the JID
+                 object set the string to "jid", otherwise leave
+                 blank for the text string.
 
-  GetFrom() - returns a string with the Jabber Identifier of the 
-              person who sent the <presence/>.
+  GetFrom()      -  returns either a string with the Jabber Identifier,
+  GetFrom("jid")    or a Net::Jabber::JID object for the person who
+                    sent the <presence/>.  To get the JID object set 
+                    the string to "jid", otherwise leave blank for the 
+                    text string.
+
+  GetEtherxTo(string) - returns the etherx:to attribute.  This is for
+                        Transport writers who need to communicate with
+                        Etherx.
+
+  GetEtherxFrom(string) -  returns the etherx:from attribute.  This is for
+                           Transport writers who need to communicate with
+                           Etherx.
 
   GetType() - returns a string with the type <presence/> this is.
 
@@ -123,19 +149,39 @@ Net::Jabber::Presence - Jabber Presence Module
 
 =head2 Creation functions
 
-  SetPresence(to=>string,         - set multiple fields in the <presence/>
-              type=>string,         at one time.  This is a cumulative
-              status=>string,       and over writing action.  If you set
-              priority=>integer,    the "to" attribute twice, the second
-              meta=>string,         setting is what is used.  If you set
-              icon=>string,         the status, and then set the priority
-              show=>string,         then both will be in the <presence/>
-              loc=>string)          tag.  For valid settings read the
-                                    specific Set functions below.
 
-  SetTo(string) - sets the to attribute.  Must be a valid Jabber Identifier 
-                  or the server will return an error message.
-                  (ie.  jabber:bob@jabber.org, etc...)
+#
+# todo: add etherxto and from to the list...
+#
+
+
+  SetPresence(to=>string|JID     - set multiple fields in the <presence/>
+              from=>string|JID,    at one time.  This is a cumulative
+              type=>string,        and over writing action.  If you set
+              status=>integer,     the "to" attribute twice, the second
+              priority=>string,    setting is what is used.  If you set
+              meta=>string,        the status, and then set the priority
+              icon=>string,        then both will be in the <presence/>
+              show=>string,        tag.  For valid settings read the
+              loc=>string)         specific Set functions below.
+
+  SetTo(string) - sets the to attribute.  You can either pass a string
+  SetTo(JID)      or a JID object.  They must be valid Jabber 
+                  Identifiers or the server will return an error message.
+                  (ie.  jabber:bob@jabber.org/Silent Bob, etc...)
+
+  SetFrom(string) - sets the from attribute.  You can either pass a string
+  SetFrom(JID)      or a JID object.  They must be valid Jabber 
+                    Identifiers or the server will return an error message.
+                    (ie.  jabber:bob@jabber.org/Silent Bob, etc...)
+
+  SetEtherxTo(string) - sets the etherx:to attribute.  This is for
+                        Transport writers who need to communicate with
+                        Etherx.
+
+  SetEtherxFrom(string) -  sets the etherx:from attribute.  This is for
+                           Transport writers who need to communicate with
+                           Etherx.
 
   SetType(string) - sets the type attribute.  Valid settings are:
 
@@ -173,9 +219,25 @@ Net::Jabber::Presence - Jabber Presence Module
                  namespace with the X module.  For more information
                  please read the documentation for Net::Jabber::X.
 
+  Reply(template=>string, - creates a new Presence object and
+        type=>string)       populates the to/from and
+                            etherxto/etherxfrom fields based
+                            the value of template.  The following
+                            templates are available:
+
+                            client: (default)
+                                 just sets the to/from
+
+                            transport:
+                            transport-reply:
+                                 the transport will send the
+                                 reply to the sender
+
+                            The type will be set in the <presence/>.
+
 =head1 AUTHOR
 
-By Ryan Eatmon in December of 1999 for http://jabber.org..
+By Ryan Eatmon in May of 2000 for http://jabber.org..
 
 =head1 COPYRIGHT
 
@@ -190,7 +252,7 @@ use strict;
 use Carp;
 use vars qw($VERSION);
 
-$VERSION = "0.8.1";
+$VERSION = "1.0";
 
 sub new {
   my $proto = shift;
@@ -201,7 +263,7 @@ sub new {
 
   bless($self, $proto);
 
-  if (@_ != ("")) {
+  if ("@_" ne ("")) {
     my @temp = @_;
     $self->{PRESENCE} = \@temp;
     my $xTree;
@@ -217,6 +279,17 @@ sub new {
 }
 
 
+##############################################################################
+#
+# GetTag - returns the Jabber tag of this object
+#
+##############################################################################
+sub GetTag {
+  my $self = shift;
+  return "presence";
+}
+
+
 ##############################################################################
 #
 # GetID - returns the id of the <presence/>.
@@ -236,7 +309,13 @@ sub GetID {
 ##############################################################################
 sub GetTo {
   my $self = shift;
-  return &Net::Jabber::GetXMLData("value",$self->{PRESENCE},"","to");
+  my ($type) = @_;
+  my $to = &Net::Jabber::GetXMLData("value",$self->{PRESENCE},"","to");
+  if ($type eq "jid") {
+    return new Net::Jabber::JID($to);
+  } else {
+    return $to;
+  }
 }
 
 
@@ -248,7 +327,37 @@ sub GetTo {
 ##############################################################################
 sub GetFrom {
   my $self = shift;
-  return &Net::Jabber::GetXMLData("value",$self->{PRESENCE},"","from");
+  my ($type) = @_;
+  my $from = &Net::Jabber::GetXMLData("value",$self->{PRESENCE},"","from");
+  if ($type eq "jid") {
+    return new Net::Jabber::JID($from);
+  } else {
+    return $from;
+  }
+}
+
+
+##############################################################################
+#
+# GetEtherxTo - returns the value of the etherx:to attribute in the 
+#               <presence/>.
+#
+##############################################################################
+sub GetEtherxTo {
+  my $self = shift;
+  return &Net::Jabber::GetXMLData("value",$self->{PRESENCE},"","etherx:to");
+}
+
+
+##############################################################################
+#
+# GetEtherxFrom - returns the value of the etherx:from attribute in the 
+#                 <presence/>.
+#
+##############################################################################
+sub GetEtherxFrom {
+  my $self = shift;
+  return &Net::Jabber::GetXMLData("value",$self->{PRESENCE},"","etherx:from");
 }
 
 
@@ -404,6 +513,9 @@ sub SetPresence {
 
   $self->SetID($presence{id}) if exists($presence{id});
   $self->SetTo($presence{to}) if exists($presence{to});
+  $self->SetFrom($presence{from}) if exists($presence{from});
+  $self->SetEtherxTo($presence{etherxto}) if exists($presence{etherxto});
+  $self->SetEtherxFrom($presence{etherxfrom}) if exists($presence{etherxfrom});
   $self->SetType($presence{type}) if exists($presence{type});
   $self->SetStatus($presence{status}) if exists($presence{status});
   $self->SetPriority($presence{priority}) if exists($presence{priority});
@@ -434,10 +546,52 @@ sub SetID {
 sub SetTo {
   my $self = shift;
   my ($to) = @_;
+  if (ref($to) eq "Net::Jabber::JID") {
+    $to = $to->GetJID();
+  }
   &Net::Jabber::SetXMLData("single",$self->{PRESENCE},"","",{to=>$to});
 }
 
 
+##############################################################################
+#
+# SetFrom - sets the from attribute in the <presence/>
+#
+##############################################################################
+sub SetFrom {
+  my $self = shift;
+  my ($from) = @_;
+  if (ref($from) eq "Net::Jabber::JID") {
+    $from = $from->GetJID();
+  }
+  &Net::Jabber::SetXMLData("single",$self->{PRESENCE},"","",{from=>$from});
+}
+
+
+##############################################################################
+#
+# SetEtherxTo - sets the etherx:to attribute in the <presence/>
+#
+##############################################################################
+sub SetEtherxTo {
+  my $self = shift;
+  my ($etherxto) = @_;
+  &Net::Jabber::SetXMLData("single",$self->{PRESENCE},"","",{"etherx:to"=>$etherxto});
+}
+
+
+##############################################################################
+#
+# SetEtherxFrom - sets the etherx:from attribute in the <presence/>
+#
+##############################################################################
+sub SetEtherxFrom {
+  my $self = shift;
+  my ($etherxfrom) = @_;
+  &Net::Jabber::SetXMLData("single",$self->{PRESENCE},"","",{"etherx:from"=>$etherxfrom});
+}
+
+
 ##############################################################################
 #
 # SetType - sets the type attribute in the <presence/>
@@ -592,6 +746,44 @@ sub MergeX {
 }
 
 
+##############################################################################
+#
+# Reply - returns a Net::Jabber::Presence object with the proper fields
+#         already populated for you.
+#
+##############################################################################
+sub Reply {
+  my $self = shift;
+  my %args;
+  while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
+
+  my $reply = new Net::Jabber::Presence();
+
+  $reply->SetID($self->GetID()) if ($self->GetID() ne "");
+  $reply->SetType(exists($args{type}) ? $args{type} : "");
+
+  if (exists($args{template})) {
+    if ($args{template} eq "transport") {
+      my $fromJID = $self->GetFrom("jid");
+      
+      $reply->SetPresence(to=>$self->GetFrom(),
+			  from=>$self->GetTo(),
+			  etherxto=>$fromJID->GetServer(),
+			  etherxfrom=>$self->GetEtherxTo(),
+			 );
+    } else {
+      $reply->SetPresence(to=>$self->GetFrom(),
+			  from=>$self->GetTo());
+    }
+  } else {
+    $reply->SetPresence(to=>$self->GetFrom(),
+			from=>$self->GetTo());
+  }
+
+  return $reply;
+}
+
+
 ##############################################################################
 #
 # debug - prints out the XML::Parser Tree in a readable format for debugging
